VERSION 5.00
Begin VB.UserControl WSlider 
   ClientHeight    =   405
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   2820
   PropertyPages   =   "ControlDemo_ShapeLabel.ctx":0000
   ScaleHeight     =   27
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   188
End
Attribute VB_Name = "WSlider"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' "Maybe" list
'   Ticks
'   DrawThumb facility with graphics
'   Add code to prevent silly values for properties

Public Enum Border_Style
    Raised
    Recessed
    Flat
    None
End Enum

Public Enum Orientation_Style
    Horizontal
    Vertical
End Enum

Private Enum Control_State
    Normal                                  ' normal display, enabled
    Disabled                                ' normal display, disabled
    Highlighted                             ' mouse over control, no mouse down
    Active                                  ' mouse over control, mouse down and captured
End Enum

Public Event Changed(Pos As Integer)        ' notify application of position change
Public Event Changing(Pos As Integer)       ' notify applicaton that position is changing

'==========================================================
'   Private Constants
'==========================================================
Private Const DEF_TRACK_WIDTH = 4           ' Default width for slider track
Private Const DEF_TRACK_INDENT = 4          ' Default indent value for track
Private Const DEF_BORDER_STYLE = None       ' Default border style for slider
Private Const DEF_THUMB_WIDTH = 20          ' Default width of the thumb
Private Const DEF_THUMB_HEIGHT = 16         ' Default height of thumb
Private Const DEF_MIN = 0                   ' Default minimum value
Private Const DEF_MAX = 100                 ' Default maximum value
Private Const DEF_SMALL_CHANGE = 1          ' Default small change value
Private Const DEF_LARGE_CHANGE = 10         ' Default large change value
Private Const DEF_ORIENTATION = Horizontal  ' Default orientation

'==========================================================
'   Private Member fields
'==========================================================
Private mTrackWidth As Integer              ' width of track
Private mTrackIndent As Integer             ' indent from control edge (+ trackwidth/2)
Private mThumbWidth As Integer              ' width of the thumb
Private mMouseLastPos As Integer            ' last recorded mouse position
Private mThumbHeight As Integer             ' height of the thumb
Private mThumbPos As Integer                ' current pixel thumb position
Private mBorderStyle As Border_Style        ' border style for the control
Private mBackColor As OLE_COLOR             ' background color (if BorderStyle = None)
Private mBackColorRGB As Long               ' RGB version of the above
Private mControlState As Control_State      ' state of the control
Private mLastChangePos As Integer           ' last Change position sent to app
Private mLastChangingPos As Integer         ' last Changing position sent to app
Private mGotFocus As Boolean                ' True if we've got the focus
Private mSmallChange As Integer             ' delta for small Slider changes
Private mLargeChange As Integer             ' delta for large Slider changes
Private mMin As Integer                     ' Minimum value for control
Private mMax As Integer                     ' Maximum value for control
Private mOrientation As Orientation_Style   ' Orientation of the control
Private mWidth As Long                      ' Logical width of control
Private mHeight As Long                     ' Logical height of control

Private Sub UserControl_Initialize()
    mThumbPos = 0
    mGotFocus = False
    mControlState = Normal
    mLastChangePos = -1                 ' use an invalid value to get a trigger
    mLastChangingPos = -1               ' ditto
End Sub

Private Sub SetLogicalSize()
    If mOrientation = Horizontal Then
        mWidth = UserControl.ScaleWidth
        mHeight = UserControl.ScaleHeight
    Else
        mWidth = UserControl.ScaleHeight
        mHeight = UserControl.ScaleWidth
    End If
End Sub

'==========================================================
'   Name:       UserControl_InitProperties
'   Purpose:    Called when a new instance of a control is
'               created.  Main job is to set private property
'               values to their declared defaults.
'==========================================================
Private Sub UserControl_InitProperties()
    mTrackWidth = DEF_TRACK_WIDTH
    mTrackIndent = DEF_TRACK_INDENT
    mBorderStyle = DEF_BORDER_STYLE
    mThumbWidth = DEF_THUMB_WIDTH
    mThumbHeight = DEF_THUMB_HEIGHT
    mSmallChange = DEF_SMALL_CHANGE
    mLargeChange = DEF_LARGE_CHANGE
    mOrientation = DEF_ORIENTATION
    mMin = DEF_MIN
    mMax = DEF_MAX
    mBackColor = UserControl.Ambient.BackColor
    OleTranslateColor mBackColor, 0, mBackColorRGB
    ' Don't call from Initialize() in case UserControl not available.
    ' Also, we need the mOrientation flag.
    SetLogicalSize
End Sub

Private Sub UserControl_Resize()
    SetLogicalSize
    ' A resize event invalidates the thumb position
    Position = 0
End Sub

'==========================================================
'   Name:       TrackRect
'   Purpose:    Return a bounding rectangle for the track
'==========================================================
Private Function TrackRect() As RECT
    TrackRect.Left = (mThumbWidth \ 2) + mTrackIndent
    TrackRect.Right = mWidth - TrackRect.Left
    TrackRect.Top = (mHeight - mTrackWidth) \ 2
    TrackRect.Bottom = TrackRect.Top + mTrackWidth
End Function

'==========================================================
'   Name:       ThumbRect
'   Purpose:    Return a bounding rectangle for the thumb
'==========================================================
Private Function ThumbRect() As RECT
    If mOrientation = Horizontal Then
        ' If it's a horizontal Slider, then position zero is on the left
        ThumbRect.Left = mThumbPos + mTrackIndent
    Else
        ' If it's a vertical Slider, then position zero is at the bottom
        ThumbRect.Left = (mWidth - mThumbWidth - (2 * mTrackIndent) - mThumbPos) + mTrackIndent
    End If
       
    ThumbRect.Right = ThumbRect.Left + mThumbWidth
    ThumbRect.Top = (mHeight - mThumbHeight) \ 2
    ThumbRect.Bottom = ThumbRect.Top + mThumbHeight
End Function

Private Sub NotifyChanging()
   If Position <> mLastChangingPos Then
       RaiseEvent Changing(Position)
       mLastChangingPos = Position
    End If
End Sub

Private Sub NotifyChange()
    If Position <> mLastChangePos Then
        RaiseEvent Changed(Position)
        mLastChangePos = Position
    End If
     
    ' Since there's been a change, the "Changing" event should get a hit too
    NotifyChanging
End Sub

'==========================================================
'   Name:       PositionFromThumb
'   Purpose:    Given a thumb location, calculate Position
'==========================================================
Private Function PositionFromThumb(ByVal Thumb As Double) As Integer
    Dim TRange As Double            ' T(hi) - T(lo)
    Dim PRange As Double            ' P(hi) - P(lo)
    
    TRange = mWidth - mThumbWidth - (2 * mTrackIndent)
    PRange = mMax - mMin
    PositionFromThumb = ((Thumb / TRange) * PRange) + mMin
End Function

'==========================================================
'   Name:       ThumbFromPosition
'   Purpose:    Given a Position, calculate thumb location
'==========================================================
Private Function ThumbFromPosition(ByVal Pos As Double) As Integer
    Dim TRange As Double            ' T(hi) - T(lo)
    Dim PRange As Double            ' P(hi) - P(lo)
    
    TRange = mWidth - mThumbWidth - (2 * mTrackIndent)
    PRange = mMax - mMin
    ThumbFromPosition = ((Pos - mMin) / PRange) * TRange
End Function

'==========================================================
'   Name:       DrawThumb
'   Purpose:    Routine to draw the default thumb
'==========================================================
Private Sub DrawThumb(hdc As MemDC, r As RECT)
    Dim Effect As Effect3D
    Dim curLeft As Integer
    
    If mControlState = Highlighted Or mControlState = Active Then
        hdc.FillRect r.Left + 1, r.Top + 1, r.Right - r.Left, r.Bottom - r.Top, 0
    End If
    
    If mControlState <> Disabled Then
        Effect = Raised
    Else
        Effect = Flat
    End If
    
    hdc.Rect3D r.Left, r.Top, r.Right - r.Left, r.Bottom - r.Top, Effect
   
    If mControlState = Highlighted Or mControlState = Active Then
        hdc.FillRect r.Left + 2, r.Top + 2, r.Right - r.Left - 4, r.Bottom - r.Top - 4, hdc.ButtonHighlightColor
    End If
    
    ' Decorate the thumb - if it's wide enough.
    curLeft = r.Left
    While r.Right - curLeft >= 4
        hdc.Rect3D curLeft + 1, r.Top + 2, 2, r.Bottom - r.Top - 4, Effect
        curLeft = curLeft + 4
    Wend
End Sub

'==========================================================
'   Name:       MoveThumb
'   Purpose:    Move the Slider thumb
'==========================================================
Private Sub MoveThumb(MouseX As Integer)
    Dim maxPos As Integer
    
    If MouseX <> mMouseLastPos Then
        ' Thumb is centred around mouse location, so subtract half thumb width & normalise
        mThumbPos = MouseX - (mThumbWidth \ 2) - mTrackIndent
        If mOrientation = Vertical Then mThumbPos = mWidth - mThumbWidth - (2 * mTrackIndent) - mThumbPos
        ' Pin the thumb at either end of travel
        If mThumbPos < 0 Then mThumbPos = 0
        maxPos = mWidth - mThumbWidth - (2 * mTrackIndent)
        If mThumbPos > maxPos Then mThumbPos = maxPos
        ' Now redraw and remember old position
        UserControl_Paint
        mMouseLastPos = MouseX
        ' Notify application that thumb is changing
        NotifyChanging
    End If
End Sub

'==========================================================
'   Name:       UserControl_KeyDown
'   Purpose:    Respond to whatever keypresses we're interested in.
'   Notes:      Can't use KeyPress, 'cos it only deals with ASCII
'               Can't use KeyUp because it ignores auto-repeat. Sigh...
'==========================================================
Private Sub UserControl_KeyDown(KeyCode As Integer, Shift As Integer)
    ' We shouldn't get a keydown unless we've got the focus and are enabled.
    ' Therefore, we only need to check that we're not currently active.
    ' Trust me - I'm a doctor..... :-)
            
    If mControlState <> Active Then
        Select Case KeyCode
            Case vbKeyLeft
                If mOrientation = Horizontal Then Position = Position - mSmallChange
            Case vbKeyUp
                If mOrientation = Vertical Then Position = Position + mSmallChange
            Case vbKeyRight
                If mOrientation = Horizontal Then Position = Position + mSmallChange
            Case vbKeyDown
                If mOrientation = Vertical Then Position = Position - mSmallChange
            Case vbKeyPageUp
                Position = Position + mLargeChange
            Case vbKeyPageDown
                Position = Position - mLargeChange
            Case vbKeyHome
                Position = mMin
            Case vbKeyEnd
                Position = mMax
        End Select
    End If
End Sub

'==========================================================
'   Name:       UserControl_MouseDown
'   Purpose:    Respond to mouse down.
'   Notes:      We have to capture the mouse for mControlState = Highlighted
'               as well as Active, otherwise we'll lose WM_MOUSEMOVE messages
'               and stay highlighted when we shouldn't be !
'==========================================================
Private Sub UserControl_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    Dim InThumb As Boolean
    Dim InTrack As Boolean
    
    If mOrientation = Vertical Then
        Dim Temp As Single
        Temp = X
        X = Y
        Y = Temp
    End If
    
    ' We're only interested if mControlState is Normal or Highlighted...  In theory,
    ' mControlState should be Highlighted at this point, but in practice, some mouse
    ' moves could have been lost because we haven't got the capture yet.
        
    If mControlState = Normal Or mControlState = Highlighted Then
        ' Figure out if mouse is over thumb or track
        InThumb = PtInRect(ThumbRect(), X, Y)
        InTrack = PtInRect(TrackRect(), X, Y)
        
        If InThumb Or InTrack Then
            ' If mControlState is Highlighted, then already got capture
            If mControlState = Normal Then API.SetCapture UserControl.hwnd
            ' Flag that we're active
            mControlState = Active
            ' If hit on track (but not thumb) then jump to new position
            If Not InThumb Then MoveThumb (X)
            mMouseLastPos = X
            UserControl_Paint
        End If
    End If
End Sub

'==========================================================
'   Name:       UserControl_MouseMove
'   Purpose:    Track mouse movement within the control
'   Notes:      We have to capture the mouse for mControlState = Highlighted
'               as well as mControlState = Active, otherwise we'll lose
'               WM_MOUSEMOVE messages and stay highlighted when we shouldn't be !
'==========================================================
Private Sub UserControl_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
    Dim InThumb As Boolean
    Dim InTrack As Boolean
  
    If mOrientation = Vertical Then
        Dim Temp As Single
        Temp = X
        X = Y
        Y = Temp
    End If
    
    ' Figure out if mouse is over thumb or track
    InThumb = PtInRect(ThumbRect(), X, Y)
    InTrack = PtInRect(TrackRect(), X, Y)
        
    Select Case mControlState
        Case Normal
            ' See if we've got a mouse-over.  If so, get highlighted !
            If InThumb Or InTrack Then
                mControlState = Highlighted
                API.SetCapture UserControl.hwnd
                UserControl_Paint
            End If
        Case Highlighted
            If InThumb = False And InTrack = False Then
                mControlState = Normal
                API.ReleaseCapture
                UserControl_Paint
            End If
        Case Active
            MoveThumb (X)
    End Select
End Sub

'==========================================================
'   Name:       UserControl_MouseUp
'   Purpose:    Respond to mouse up.
'   Notes:      We should only respond to a mouse up message if mControlState
'               is Active.
'==========================================================
Private Sub UserControl_MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
    Dim InThumb As Boolean
    Dim InTrack As Boolean
    
    If mOrientation = Vertical Then
        Dim Temp As Single
        Temp = X
        X = Y
        Y = Temp
    End If
    
    If mControlState = Active Then
        ' Figure out if mouse is over thumb or track
        InThumb = PtInRect(ThumbRect(), X, Y)
        InTrack = PtInRect(TrackRect(), X, Y)
        
        ' Note - if mouse is still inside control, then we switch state to
        ' Highlighted and keep the capture.  If not, we switch state to Normal
        ' Theoretically, the former should always be true, but let's be defensive.
        
        If InThumb Or InTrack Then
            mControlState = Highlighted
        Else
            mControlState = Normal
            API.ReleaseCapture
        End If
        
        UserControl_Paint
        NotifyChange
    End If
End Sub

'==========================================================
'   Name:       UserControl_GotFocus
'   Purpose:    Called when we get the focus.  Just remember the
'               fact and repaint the control with focus
'==========================================================
Private Sub UserControl_GotFocus()
    mGotFocus = True
    UserControl_Paint
End Sub

'==========================================================
'   Name:       UserControl_LostFocus
'   Purpose:    Called when we lose the focus.
'==========================================================
Private Sub UserControl_LostFocus()
    mGotFocus = False
    UserControl_Paint
End Sub

'==========================================================
'   Name:       UserControl_Paint
'   Purpose:    Draw the control image on an off-screen
'               bitmap and then blit directly onto screen.
'==========================================================
Private Sub UserControl_Paint()
    Dim r As RECT
    Dim hdc As MemDC
    Dim Effect As Effect3D
   
    ' Init a memory device context for painting
    Set hdc = New MemDC
    hdc.Initialize UserControl.hdc, mWidth, mHeight, mOrientation = Vertical
    
    ' First job is to draw the control background
    
    If mBorderStyle <> None Then
        hdc.Rect3D 0, 0, mWidth, mHeight, mBorderStyle
    Else
        hdc.FillRect 0, 0, mWidth, mHeight, mBackColorRGB
    End If
    
    ' Now we need to draw the slider "track"
    r = TrackRect
    
    If mGotFocus And (mControlState <> Active) Then
        Effect = Raised
        hdc.FillRect r.Left + 1, r.Top + 1, r.Right - r.Left, r.Bottom - r.Top, 0
    Else
        Select Case mControlState
            Case Normal
                Effect = Recessed
            Case Disabled
                Effect = Flat
            Case Highlighted
                Effect = Raised
            Case Active
                Effect = Selected
        End Select
    End If
    
    hdc.Rect3D r.Left, r.Top, r.Right - r.Left, r.Bottom - r.Top, Effect
    
    ' Next, draw the thumb
    DrawThumb hdc, ThumbRect
        
    ' Blit the image to the display
    hdc.Blit
End Sub

'==========================================================
'   Name:       UserControl_ReadProperties
'   Purpose:    Read control properties from PropBag
'==========================================================
Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
    On Error Resume Next
    With PropBag
        mTrackWidth = .ReadProperty("TrackWidth", DEF_TRACK_WIDTH)
        mTrackIndent = .ReadProperty("TrackIndent", DEF_TRACK_INDENT)
        mBorderStyle = .ReadProperty("BorderStyle", DEF_BORDER_STYLE)
        mThumbWidth = .ReadProperty("ThumbWidth", DEF_THUMB_WIDTH)
        mThumbHeight = .ReadProperty("ThumbHeight", DEF_THUMB_HEIGHT)
        mBackColor = .ReadProperty("BackColor", UserControl.Ambient.BackColor)
        OleTranslateColor mBackColor, 0, mBackColorRGB
        mMin = .ReadProperty("Min", DEF_MIN)
        mMax = .ReadProperty("Max", DEF_MAX)
        mSmallChange = .ReadProperty("SmallChange", DEF_SMALL_CHANGE)
        mLargeChange = .ReadProperty("LargeChange", DEF_LARGE_CHANGE)
        UserControl.Enabled = .ReadProperty("Enabled", True)
        If Not UserControl.Enabled Then mControlState = Disabled
        mOrientation = .ReadProperty("Orientation", DEF_ORIENTATION)
    End With
End Sub

'==========================================================
'   Name:       UserControl_WriteProperties
'   Purpose:    Write control properties to PropBag
'==========================================================
Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
    On Error Resume Next
    With PropBag
        .WriteProperty "TrackWidth", mTrackWidth, DEF_TRACK_WIDTH
        .WriteProperty "TrackIndent", mTrackIndent, DEF_TRACK_INDENT
        .WriteProperty "BorderStyle", mBorderStyle, DEF_BORDER_STYLE
        .WriteProperty "ThumbWidth", mThumbWidth, DEF_THUMB_WIDTH
        .WriteProperty "ThumbHeight", mThumbHeight, DEF_THUMB_HEIGHT
        .WriteProperty "BackColor", mBackColor, UserControl.Ambient.BackColor
        .WriteProperty "Min", mMin, DEF_MIN
        .WriteProperty "Max", mMax, DEF_MAX
        .WriteProperty "SmallChange", mSmallChange, DEF_SMALL_CHANGE
        .WriteProperty "LargeChange", mLargeChange, DEF_LARGE_CHANGE
        .WriteProperty "Enabled", UserControl.Enabled, True
        .WriteProperty "Orientation", mOrientation, DEF_ORIENTATION
    End With
End Sub

Public Property Get TrackWidth() As Integer
Attribute TrackWidth.VB_Description = "Sets the width of the slider control's track bar."
    TrackWidth = mTrackWidth
End Property

Public Property Let TrackWidth(ByVal NewTrackWidth As Integer)
    If mTrackWidth <> NewTrackWidth And NewTrackWidth >= 2 Then
        mTrackWidth = NewTrackWidth
        PropertyChanged "TrackWidth"
        UserControl_Paint
   End If
End Property

Public Property Get BorderStyle() As Border_Style
Attribute BorderStyle.VB_Description = "Sets the type of border which surrounds the control."
    BorderStyle = mBorderStyle
End Property

Public Property Let BorderStyle(ByVal NewBorderStyle As Border_Style)
    If mBorderStyle <> NewBorderStyle Then
        mBorderStyle = NewBorderStyle
        PropertyChanged "BorderStyle"
        UserControl_Paint
    End If
End Property

Public Property Get BackColor() As OLE_COLOR
Attribute BackColor.VB_Description = "Background Color for Slider control"
Attribute BackColor.VB_UserMemId = -501
    BackColor = mBackColor
End Property

Public Property Let BackColor(ByVal NewBackColor As OLE_COLOR)
    If mBackColor <> NewBackColor Then
        mBackColor = NewBackColor
        PropertyChanged "BackColor"
        OleTranslateColor mBackColor, 0, mBackColorRGB
        ' Setting BackColor forces BorderStyle to 'none'
        mBorderStyle = None
        UserControl_Paint
    End If
End Property

Public Property Get ThumbWidth() As Integer
Attribute ThumbWidth.VB_Description = "Set the width of the Slider control's thumb."
    If mOrientation = Horizontal Then
        ThumbWidth = mThumbWidth
    Else
        ThumbWidth = mThumbHeight
    End If
End Property

Public Property Let ThumbWidth(ByVal NewThumbWidth As Integer)
    If mOrientation = Horizontal Then
        If mThumbWidth <> NewThumbWidth Then
            mThumbWidth = NewThumbWidth
            PropertyChanged "ThumbWidth"
            UserControl_Paint
        End If
    Else
        If mThumbHeight <> NewThumbWidth Then
            mThumbHeight = NewThumbWidth
            PropertyChanged "ThumbHeight"
            UserControl_Paint
        End If
    End If
End Property

Public Property Get ThumbHeight() As Integer
Attribute ThumbHeight.VB_Description = "Set the height of the Slider control's thumb."
    If mOrientation = Horizontal Then
        ThumbHeight = mThumbHeight
    Else
        ThumbHeight = mThumbWidth
    End If
End Property

Public Property Let ThumbHeight(ByVal NewThumbHeight As Integer)
    If mOrientation = Horizontal Then
        If mThumbHeight <> NewThumbHeight Then
            mThumbHeight = NewThumbHeight
            PropertyChanged "ThumbHeight"
            UserControl_Paint
        End If
    Else
        If mThumbWidth <> NewThumbHeight Then
            mThumbWidth = NewThumbHeight
            PropertyChanged "ThumbWidth"
            UserControl_Paint
        End If
    End If
End Property

Public Property Get TrackIndent() As Integer
Attribute TrackIndent.VB_Description = "Allows you to indent the Slider's track away from the edge of the control."
    TrackIndent = mTrackIndent
End Property

Public Property Let TrackIndent(ByVal NewTrackIndent As Integer)
    If mTrackIndent <> NewTrackIndent Then
        mTrackIndent = NewTrackIndent
        PropertyChanged "TrackIndent"
        UserControl_Paint
    End If
End Property

Public Property Get Enabled() As Boolean
Attribute Enabled.VB_Description = "Enable or disables the control."
Attribute Enabled.VB_UserMemId = -514
    Enabled = UserControl.Enabled
End Property

Public Property Let Enabled(ByVal NewEnabled As Boolean)
    If (UserControl.Enabled) <> NewEnabled Then
        UserControl.Enabled = NewEnabled
        
        If NewEnabled Then
            mControlState = Normal
        Else
            mControlState = Disabled
        End If
        
        PropertyChanged "Enabled"
        UserControl_Paint
    End If
End Property

Public Property Get Min() As Integer
Attribute Min.VB_Description = "Minimum possible value for the Slider control."
    Min = mMin
End Property

Public Property Let Min(ByVal NewMin As Integer)
    If mMin <> NewMin And NewMin <= mMax Then
        mMin = NewMin
        ' Changing mMin invalidates thumb position
        mThumbPos = 0
        PropertyChanged "Min"
        UserControl_Paint
    End If
End Property

Public Property Get Max() As Integer
Attribute Max.VB_Description = "Maximum possible value for the Slider control."
    Max = mMax
End Property

Public Property Let Max(ByVal NewMax As Integer)
    If mMax <> NewMax And NewMax >= mMin Then
        mMax = NewMax
        ' Changing mMax invalidates thumb position
        mThumbPos = 0
        PropertyChanged "Max"
        UserControl_Paint
    End If
End Property

Public Property Get Position() As Integer
Attribute Position.VB_Description = "Current position of the Slider."
Attribute Position.VB_MemberFlags = "200"
    Position = PositionFromThumb(mThumbPos)
End Property

Public Property Let Position(ByVal NewPosition As Integer)
    ' Firstly, pin the position to the Min, Max bounds
    If NewPosition < mMin Then NewPosition = mMin
    If NewPosition > mMax Then NewPosition = mMax
    ' Has anything changed ?
    If PositionFromThumb(mThumbPos) <> NewPosition Then
        mThumbPos = ThumbFromPosition(NewPosition)
        UserControl_Paint
        NotifyChange
    End If
End Property

Public Property Get SmallChange() As Integer
Attribute SmallChange.VB_Description = "Change in Slider position when Left/Right/Up/Down cursor keys are pressed."
    SmallChange = mSmallChange
End Property

Public Property Let SmallChange(ByVal NewSmallChange As Integer)
    ' For maximum versatility, there are no restrictions here.
    ' Setting SmallChange to zero will disable left/right/up/down cursor keys.
    ' Setting SmallChange < 0 reverses the 'sense' of cursor keys.
    mSmallChange = NewSmallChange
End Property

Public Property Get LargeChange() As Integer
Attribute LargeChange.VB_Description = "Change in Slider Position when PgUp/PgDn keys are pressed."
    LargeChange = mLargeChange
End Property

Public Property Let LargeChange(ByVal NewLargeChange As Integer)
    ' For maximum versatility, there are no restrictions here.
    ' Setting LargeChange to zero will disable PgUp/PgDn cursor keys.
    ' Setting LargeChange < 0 reverses the 'sense' of PgUp/PgDn.
    mLargeChange = NewLargeChange
End Property

Public Property Get Orientation() As Orientation_Style
Attribute Orientation.VB_Description = "Specify whether control is horizontal or vertical."
    Orientation = mOrientation
End Property

Public Property Let Orientation(ByVal NewOrientation As Orientation_Style)
    Dim Temp As Integer

    If mOrientation <> NewOrientation Then
        ' Swap width and height
        Temp = Height
        Height = Width
        Width = Temp
        mOrientation = NewOrientation
        SetLogicalSize
        UserControl_Paint
    End If
End Property

