VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "MemDC"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'======================================================================
'   File:       MEMDC.CLS
'   Purpose:    This class implements a simple-minded off-screen bitmap
'               capability through an API-level memory device context.
'               MFC it ain't :-).  Based on an idea by Microsoft.
'               Dave Jewell, 1997 for Wrox Press.
'======================================================================

Option Explicit

Public Enum Effect3D
    Raised
    Recessed
    Flat
    Selected
End Enum

'======================================================================
' Private Data Members
'======================================================================
Private btnFace As Long         ' color to use for the 3d face
Private btnHighlight As Long    ' color to use for the 3d highlight
Private btnShadow As Long       ' color to use for the 3d shadow
Private hMemDC As Long          ' memory device context
Private hCtlDC As Long          ' display device context for the control
Private hOldBitmap As Long      ' original bitmap handle for the memory DC
Private ControlWidth As Long    ' width of the control - must be PIXELS !
Private ControlHeight As Long   ' height of the control - must be PIXELS !
Private XYSkew As Boolean       ' skew coordinates ?
Private SkewCount As Integer    ' reentrancy skew count

'======================================================================
'   Name:       Initialize
'   Purpose:    Init the off-screen DC and set up private variables
'======================================================================
Public Sub Initialize(CtlHdc As Long, CtlWidth As Long, CtlHeight As Long, Optional Skew As Boolean = False)
    If CtlHdc <> 0 And CtlWidth <> 0 And CtlHeight <> 0 And hMemDC = 0 Then
        ' Stash the control's device context - need for Blit()
        hCtlDC = CtlHdc
        ' Stash the skew value
        XYSkew = Skew
        SkewCount = 0
        ' Stash the width and height of the control
        If Not XYSkew Then
            ControlWidth = CtlWidth
            ControlHeight = CtlHeight
        Else
            ControlWidth = CtlHeight
            ControlHeight = CtlWidth
        End If
        
        ' Create the memory device context
        hMemDC = CreateCompatibleDC(hCtlDC)
        ' Create a bitmap, select it into the memory DC and stash the old bitmap
        hOldBitmap = SelectObject(hMemDC, CreateCompatibleBitmap(hCtlDC, ControlWidth, ControlHeight))
        ' Set up color metrics for 3D surface rendering
        btnFace = GetSysColor(COLOR_BTNFACE)
        btnHighlight = GetSysColor(COLOR_BTNHIGHLIGHT)
        btnShadow = GetSysColor(COLOR_BTNSHADOW)
    End If
End Sub

Private Sub SkewBegin(nLeft As Long, nTop As Long, nWidth As Long, nHeight As Long)
    Dim Temp As Long
    SkewCount = SkewCount + 1
    If SkewCount = 1 And XYSkew Then
        Temp = nLeft
        nLeft = nTop
        nTop = Temp
        Temp = nWidth
        nWidth = nHeight
        nHeight = Temp
    End If
End Sub

Private Sub SkewEnd()
    SkewCount = SkewCount - 1
End Sub
'======================================================================
'   Name:       Class_Terminate()
'   Purpose:    Clean-up code called when a class object is destroyed.
'======================================================================
Private Sub Class_Terminate()
    If hMemDC <> 0 Then
        ' Put back the old bitmap, and nuke the memory bitmap
        DeleteObject (SelectObject(hMemDC, hOldBitmap))
        ' Also need to nuke the memory DC itself
        DeleteDC hMemDC
    End If
End Sub

'======================================================================
' Name:     Blit
' Purpose:  Blit the memory DC onto the display
'======================================================================
Public Sub Blit()
    BitBlt hCtlDC, 0, 0, ControlWidth, ControlHeight, hMemDC, 0, 0, SRCCOPY
End Sub

'======================================================================
'   Name:       Get ButtonFaceColor()
'   Purpose:    Returns button face color for interested parties
'======================================================================
Public Property Get ButtonFaceColor() As Long
    ButtonFaceColor = btnFace
End Property

'======================================================================
'   Name:       Get ButtonHighlightColor()
'   Purpose:    Returns button highlight color for interested parties
'======================================================================
Public Property Get ButtonHighlightColor() As Long
    ButtonHighlightColor = btnHighlight
End Property

'======================================================================
'   Name:       Get ButtonShadowColor()
'   Purpose:    Returns button shadow color for interested parties
'======================================================================
Public Property Get ButtonShadowColor() As Long
    ButtonShadowColor = btnShadow
End Property

'======================================================================
'   Name:       FillRect
'   Purpose:    Fast rectangle fill using ExtTextOut
'======================================================================
Public Sub FillRect(ByVal nLeft As Long, ByVal nTop As Long, ByVal nWidth As Long, ByVal nHeight As Long, rgbColor As Long)
    Dim r As RECT
    
    SkewBegin nLeft, nTop, nWidth, nHeight
    If hMemDC <> 0 Then
        ' Set wanted background color
        SetBkColor hMemDC, rgbColor
        ' Init rectangle dimensions
        r.Left = nLeft
        r.Top = nTop
        r.Right = nWidth + nLeft
        r.Bottom = nHeight + nTop
        ' Write to memory DC using ExtTextOut
        ExtTextOut hMemDC, 0, 0, 6, r, "", 0, 0
    End If
    SkewEnd
End Sub

'======================================================================
'   Name:       Rect3D
'   Purpose:    Draw a 3D shaded rectangle on our memory DC.
'======================================================================
Public Sub Rect3D(ByVal nLeft As Long, ByVal nTop As Long, ByVal nWidth As Long, ByVal nHeight As Long, FX As Effect3D)
    Dim LR_color As Long
    Dim UL_color As Long
   
    SkewBegin nLeft, nTop, nWidth, nHeight
    If hMemDC <> 0 Then
        ' Set upper-left and lower-right color values as wanted
        Select Case FX
            Case Flat
                LR_color = btnShadow
                UL_color = btnShadow
            Case Raised
                LR_color = btnShadow
                UL_color = btnHighlight
            Case Recessed
                LR_color = btnHighlight
                UL_color = btnShadow
            Case Selected
                LR_color = btnHighlight
                UL_color = vbBlack
        End Select
    
        ' The rectangle is rendered using three calls to FillRect.  It would
        ' probably be more efficient to draw a series of non-overlapping
        ' single-pixel lines directly with ExtTextOut, but this code is
        ' adequate for now.
        
        Me.FillRect nLeft, nTop, nWidth, nHeight, LR_color
        nWidth = nWidth - 1
        nHeight = nHeight - 1
        Me.FillRect nLeft, nTop, nWidth, nHeight, UL_color
        nLeft = nLeft + 1
        nTop = nTop + 1
        nWidth = nWidth - 1
        nHeight = nHeight - 1
        Me.FillRect nLeft, nTop, nWidth, nHeight, btnFace

        ' The Selected style is simply a colour inversion.
        If FX = Selected Then InvertRect nLeft, nTop, nWidth, nHeight
    End If
    SkewEnd
End Sub

'======================================================================
' Name:     InvertRect
' Purpose:  Invert a designated rectangle
'======================================================================
Public Sub InvertRect(ByVal nLeft As Long, ByVal nTop As Long, ByVal nWidth As Long, ByVal nHeight As Long)
    Dim rc As RECT
   
    SkewBegin nLeft, nTop, nWidth, nHeight
    rc.Left = nLeft
    rc.Top = nTop
    rc.Right = nLeft + nWidth
    rc.Bottom = nTop + nHeight
    API.InvertRect hMemDC, rc
    SkewEnd
End Sub
